/**
 * This file Copyright (c) 2018 Magnolia International
 * Ltd.  (http://www.magnolia-cms.com). All rights reserved.
 *
 *
 * This file is dual-licensed under both the Magnolia
 * Network Agreement and the GNU General Public License.
 * You may elect to use one or the other of these licenses.
 *
 * This file is distributed in the hope that it will be
 * useful, but AS-IS and WITHOUT ANY WARRANTY; without even the
 * implied warranty of MERCHANTABILITY or FITNESS FOR A
 * PARTICULAR PURPOSE, TITLE, or NONINFRINGEMENT.
 * Redistribution, except as permitted by whichever of the GPL
 * or MNA you select, is prohibited.
 *
 * 1. For the GPL license (GPL), you can redistribute and/or
 * modify this file under the terms of the GNU General
 * Public License, Version 3, as published by the Free Software
 * Foundation.  You should have received a copy of the GNU
 * General Public License, Version 3 along with this program;
 * if not, write to the Free Software Foundation, Inc., 51
 * Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * 2. For the Magnolia Network Agreement (MNA), this file
 * and the accompanying materials are made available under the
 * terms of the MNA which accompanies this distribution, and
 * is available at http://www.magnolia-cms.com/mna.html
 *
 * Any modifications to this file must keep this entire header
 * intact.
 *
 */
package info.magnolia.ui.vaadin.gwt.client.sticker.widget;

import com.google.gwt.dom.client.DivElement;
import com.google.gwt.dom.client.SpanElement;
import com.google.gwt.dom.client.Style;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.HasClickHandlers;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.ui.Widget;

/**
 * Client-side implementation for the {@link info.magnolia.ui.vaadin.sticker.Sticker} component.
 */
public class VSticker extends Widget implements HasClickHandlers {

    private static final String STICKER_CLASS_NAME = "v-sticker";

    private static final String STICKER_ENVIRONMENT_CLASS_NAME = "v-sticker-environment";

    private static final String STICKER_WEBAPP_CLASS_NAME = "v-sticker-webapp";

    private DivElement div = DivElement.as(DOM.createDiv());

    private SpanElement environmentSpan = SpanElement.as(DOM.createSpan());

    private SpanElement webappSpan = SpanElement.as(DOM.createSpan());

    private String color;

    private String webappName;

    private String environment;

    private boolean expanded = false;

    public VSticker() {
        DOM.appendChild(div, environmentSpan);
        DOM.appendChild(div, webappSpan);

        div.addClassName(STICKER_CLASS_NAME);
        environmentSpan.addClassName(STICKER_ENVIRONMENT_CLASS_NAME);
        webappSpan.addClassName(STICKER_WEBAPP_CLASS_NAME);

        addClickHandler(handler -> {
            if (expanded) {
                webappSpan.getStyle().setDisplay(Style.Display.NONE);
                expanded = false;
            } else {
                webappSpan.getStyle().setDisplay(Style.Display.INLINE_BLOCK);
                expanded = true;
            }
        });

        setElement(div);
    }

    public void setEnvironment(String environment) {
        this.environment = environment;
        environmentSpan.setInnerText(getEnvironment());
    }

    public void setWebappName(String webappName) {
        this.webappName = webappName;
        webappSpan.setInnerText(getWebappName());
    }

    public void setColor(String color) {
        this.color = color;
        div.getStyle().setBackgroundColor(getColor());
    }

    public String getEnvironment() {
        return environment;
    }

    public String getWebappName() {
        return webappName;
    }

    public String getColor() {
        return color;
    }

    @Override
    public HandlerRegistration addClickHandler(ClickHandler handler) {
        return addDomHandler(handler, ClickEvent.getType());
    }
}
